/*
 * FICS-Extensions used at GICS chess.unix-ag.uni-kl.de 5000
 * (c) 1995/1996 by Ulrich Schlechte <Ulrich.Schlechte@ipp.tu-clausthal.de>
 *		    Klaus Knopper <Knopper@unix-ag.uni-kl.de>
 *
 * Free for Non-Commercial use (GPL)
 *
 *
 *
*/

#include "stdinclude.h"

#include "config.h"
#include "common.h"
#include "utils.h"
#include "command.h"
#include "playerdb.h"
#include "gamedb.h"
#include "gameproc.h"
#include "comproc.h"
#include "movecheck.h"
#include "network.h"
#include "stdinclude.h"
#include <errno.h>

/* USTAT */
PUBLIC userstat_type userstat;

PUBLIC void init_userstat()
{
  FILE *fp;
  char name[120];
  int i;

  sprintf(name, "%s/user.stats", stats_dir);
  if ((fp = fopen(name, "r")) == NULL) {
    bzero((char *) &userstat, sizeof(userstat));
  } else {
    for (i = 0; i < 48; i++)
      fscanf(fp, "%d\n", &userstat.users[i]);
    fscanf(fp, "%d\n", &userstat.usermax);
    fscanf(fp, "%d\n", (int *) &userstat.usermaxtime);
    fscanf(fp, "%d\n", &userstat.logins);
    fscanf(fp, "%d\n", &userstat.games);
    fscanf(fp, "%d\n", &userstat.gamemax);
    fscanf(fp, "%d\n", (int *) &userstat.gamemaxtime);
    fclose(fp);
  }
}

PUBLIC void save_userstat()
{
  FILE *fp;
  char name[120];
  int i;

  sprintf(name, "%s/user.stats", stats_dir);
  if ((fp = fopen(name, "w")) == NULL)
    return;
  for (i = 0; i < 48; i++)
    fprintf(fp, "%d\n", userstat.users[i]);
  fprintf(fp, "%d\n%d\n%d\n", userstat.usermax, userstat.usermaxtime, userstat.logins);
  fprintf(fp, "%d\n%d\n%d\n", userstat.games, userstat.gamemax, userstat.gamemaxtime);
  fclose(fp);
}

PUBLIC int com_ustat(int p, param_list param)
{
  int i, j, s, max = 15, scale = 5, avg = 0;
  int mapping[48];
  time_t now;
  struct tm *nowtm;

  time(&now);
  nowtm = localtime(&now);

  s = nowtm->tm_hour * 2;
  if (nowtm->tm_min > 30)
    s++;
  j = 0;
  for (i = s + 1; i < 48; i++) {
    mapping[j] = i;
    max = MAX(userstat.users[i], max);
    avg += userstat.users[i];
    j++;
  }
  for (i = 0; i <= s; i++) {
    mapping[j] = i;
    max = MAX(userstat.users[i], max);
    avg += userstat.users[i];
    j++;
  }
  scale = max / 15;
  max = scale * 15;
  s = 1;
  if (nowtm->tm_min > 30)
    s = 0;
  pprintf(p, "\n");
  pprintf(p, "Statistics the last 24 hours. (#=%d players)\n", scale);
  pprintf(p, "     | ");
  for (i = 0; i < 48; i++)
    pprintf(p, "%c", (userstat.users[mapping[i]] > max) ? '*' : ' ');
  pprintf(p, "\n");
  for (j = 15; j > 0; j--) {
    pprintf(p, " %3d | ", j * scale);
    for (i = 0; i < 48; i++) {
      if (j != 1) {
	pprintf(p, "%c", (userstat.users[mapping[i]] >= j * scale) ? '#' : ' ');
      } else {
	pprintf(p, "%c", (userstat.users[mapping[i]] >= j * scale) ? '#' : (userstat.users[mapping[i]] > 1 ? '.' : ' '));
      }
    }
    pprintf(p, "\n");
  }
  pprintf(p, "     ");
  for (i = 0; i < 50; i++)
    pprintf(p, "-");
  pprintf(p, "\n");
  pprintf(p, "       ");
  if (s == 1)
    pprintf(p, " ");
  for (i = 0; i < 24; i++)
    pprintf(p, "%c ", (mapping[i * 2 + s] / 2 < 10) ? (char) ((mapping[i * 2 + s] / 2) % 10) + '0' : (char) ((mapping[i * 2 + s] / 2) / 10) + '0');
  pprintf(p, "\n");
  pprintf(p, "       ");
  if (s == 1)
    pprintf(p, " ");
  for (i = 0; i < 24; i++)
        pprintf(p, "%c ", (mapping[i * 2 + s] / 2 > 9) ? (char) ((mapping[i * 2 + s] / 2) % 10) + '0' : ' ');
  pprintf(p, "\n    TIME");
  pprintf(p, "\n\n");
  pprintf(p, "Highest user count: %d Players at %s\n", userstat.usermax, strltime(&userstat.usermaxtime));
  pprintf(p, "Highest game count: %d Games at %s\n", userstat.gamemax, strltime(&userstat.gamemaxtime));
  pprintf(p, "Total logins: %d Total Games played: %d\n", userstat.logins, userstat.games);
  pprintf(p, "\n");
  return COM_OK;
}

PUBLIC int com_tourney(int p, param_list param)
{
  int retval;

  ASSERT(param[0].type == TYPE_NULL);
  if ((retval = pcommand(p, "set tourney")) != COM_OK)
    return retval;
  else
    return COM_OK_NOPROMPT;
}

#if 0
PUBLIC int com_reply(int p, param_list param)
{
  if (parray[p].last_reply < 0) {
    pprintf(p, "No one to reply to!\n");
    return COM_OK;
  }
  return tell(p, parray[p].last_reply, param[0].val.string, 0, 0);
}
#endif

#if 0
PUBLIC int com_ping(int p, param_list param)
{
  int p1;
  struct timeval timeState;
  struct timezone tz;

#ifdef TIMESEAL
  if ((p1 = player_find_bylogin(param[0].val.word)) < 0) {
    pprintf(p, "Player \"%s\" not logged on!\n", param[0].val.word);
    return COM_OK;
  }
  if ((!con[parray[p1].socket].timeseal) || (con[parray[p1].socket].method)) {
    pprintf(p, "%s is not using timeseal or uses an old client.\n", parray[p1].name);
    return COM_OK;
  }
  pprintf(p1, "\n[P]\n");
  gettimeofday(&timeState, &tz);
  con[parray[p1].socket].pingtime = (((int) timeState.tv_sec % 10000) * 1000) + ((int) timeState.tv_usec / 1000);
  con[parray[p1].socket].pingplayer = p;
#endif
  return COM_OK;
}
#endif

/*PUBLIC void dprintf(char *format,...)
{
  char tmp[10 * MAX_LINE_SIZE];*/
/* Make sure you can handle 10 lines worth of
				 * stuff */
/*
  int retval;
  time_t now;
  va_list ap;

  va_start(ap, format);

  retval = vsprintf(tmp, format, ap);
  if (strlen(tmp) > 10 * MAX_LINE_SIZE) {
    return;
  }
  time(&now);
  if (tmp[strlen(tmp) - 1] != '\n') {
    fprintf(stderr, "%s %s\n", strltime(&now), tmp);
  } else {
    fprintf(stderr, "%s %s", strltime(&now), tmp);
  }
  va_end(ap);
  return;
}*/

#if 0
PUBLIC void adminlog(int p, char *command, char *parameter)
{
  FILE *fp;
  char fname[128];
  time_t t;

  time(&t);
  sprintf(fname, "%s/admin.log", stats_dir);
  if ((fp = fopen(fname, "a")) == NULL) {
    dprintf("Error opening %s", fname);
    return;
  }
  fprintf(fp, "%s [%s] %s %s\n", strltime(&t), parray[p].name, command, parameter);
  fclose(fp);
}
#endif

#if 0
PUBLIC int com_ataillog(int p, param_list param)
{
  char tmp[128];

  ASSERT(parray[p].adminLevel > ADMIN_ADMIN);
  sprintf(tmp, "tail -20 %s/admin.log", stats_dir);
  psend_command(p, tmp, NULL);
  return COM_OK;
}
#endif

PUBLIC void game_save_playerratio(char *file, char *Opponent, int Result, int rated)
{
  FILE *fp, *fp1;
  char Name[32], file1[256];
  int win, loss, draw, flag = 0, inc = 1;

  if (!rated)
    inc = 100000;

  sprintf(file1, "%s.new", file);
  if ((fp = fopen(file, "r")) != NULL) {
    fp1 = fopen(file1, "w");
    while (!feof(fp)) {
      if (fscanf(fp, "%s %d %d %d\n", Name, &win, &draw, &loss) != 4) {
	break;
      }
      if (strcasecmp(Name, Opponent) == 0) {
	if (Result == 1)
	  win += inc;
	if (Result == 0)
	  loss += inc;
	if (Result == -1)
	  draw += inc;
	flag++;
      }
      fprintf(fp1, "%s %d %d %d\n", Name, win, draw, loss);
    }
    fclose(fp);
    if (!flag) {
      win = loss = draw = 0;
      if (Result == 1)
	win += inc;
      if (Result == 0)
	loss += inc;
      if (Result == -1)
	draw += inc;
      fprintf(fp1, "%s %d %d %d\n", Opponent, win, draw, loss);
    }
    fclose(fp1);
    unlink(file);
    if (rename(file1, file) != 0) {
      pprintf("FICS: Problems rename %s to %s : %d\n", file1, file, errno);
      return;
    }
    return;
  } else {
    fp = fopen(file, "w");
    win = loss = draw = 0;
    if (Result == 1)
      win += inc;
    if (Result == 0)
      loss += inc;
    if (Result == -1)
      draw += inc;
    fprintf(fp, "%s %d %d %d\n", Opponent, win, draw, loss);
    fclose(fp);
  }
}

PUBLIC int com_pstat(int p, param_list param)
{
  int p1 = p, p2, ptemp, pflag=0;
  int p1_connected = 1, p2_connected = 1;
  int win = 0, draw = 0, loss = 0, flag = 0;
  FILE *fp;
  char Name[32], fname[256];

  if (param[0].type == TYPE_NULL) {
    if (parray[p].game < 0) {
      pprintf(p, "You are not playing a game.\n");
      return COM_OK;
    }
    if (garray[parray[p].game].status == GAME_EXAMINE) {
      pprintf(p, "You are examining a game please specify variables.\n");
      return COM_OK;
    }
    p2 = parray[p].opponent;
  } else {
    if (!FindPlayer(p, param[0].val.word, &p2, &p2_connected)) {
      pprintf(p, "No user named \"%s\" is logged in.\n", param[0].val.word);
      return COM_OK;
    }
    if (param[1].type != TYPE_NULL) {
      if (!FindPlayer(p, param[1].val.word, &p1, &p1_connected)) {
	pprintf(p, "No user named \"%s\" is logged in.\n", param[1].val.word);
	if (!p2_connected)
	  player_remove(p2);
	return COM_OK;
      }
      ptemp = p2;
      p2 = p1;
      p1 = ptemp;
      pflag = 1;
    }
  }
  if (p1 == p2) {
    pprintf(p, "You have not played against yourself ;-)\n");
    if (!p1_connected)
      player_remove(p1);
    if (!p2_connected)
      player_remove(p2);
    return COM_OK;
  }
  sprintf(fname, "%s/player_data/%c/%s.gstats", stats_dir, parray[p1].login[0], parray[p1].login);
  if ((fp = fopen(fname, "r")) != NULL) {
    while (!feof(fp)) {
      if (fscanf(fp, "%s %d %d %d\n", Name, &win, &draw, &loss) != 4) {
	win = loss = draw = 0;
	break;
      }
      if (strcasecmp(Name, parray[p2].name) == 0) {
	flag++;
	break;
      }
    }
    fclose(fp);
  }
  if (!flag)
    win = loss = draw = 0;
  pprintf(p, "Record for %s vs. %s:\n", parray[p1].name, parray[p2].name);
  pprintf(p, "                            wins     losses     draws\n");
  pprintf(p, "                rated       %4d       %4d      %4d\n", win % 100000, loss % 100000, draw % 100000);
  pprintf(p, "              unrated       %4d       %4d      %4d\n", win / 100000, loss / 100000, draw / 100000);
  /*  pprintf(p, "Ratio of %s vs. %s (r/u): %d/%d wins %d/%d draws %d/%d losses\n", parray[p1].name, parray[p2].name,
   *	  win % 100000, win / 100000, draw % 100000, draw / 100000, loss % 100000, loss / 100000);
   */
  if (pflag) {
    ptemp = p1;
    p1 = p2;
    p2 = ptemp;
  }
  if (!p1_connected)
    player_remove(p1);
  if (!p2_connected)
    player_remove(p2);
  return COM_OK;
}

#if 0
PUBLIC void tellobservers(int g, char *s, int f)
{
  int p1;
  char tmp[200];

  for (p1 = 0; p1 < p_num; p1++) {
    if (parray[p1].status != PLAYER_PROMPT)
      continue;
    if ((player_is_observe(p1, g)) || (f && (parray[p1].game == g))) {
      pprintf_prompt(p1, "\nGame %d: %s", g + 1, s);
    }
  }
#ifdef ROBSERVE
  if (garray[g].exported) {
    sprintf(tmp, "REMOTEGAME: %s", s);
    iic_sendgameinfo(g + 1, 1, tmp);
  }
  return;
#endif
}
#endif








