/* 
  matchproc.c
  Feb 9 1996 - rewritten - DAV
*/

#include "matchproc.h"
#include "gameproc.h"
#include "formula.h"
#include "rmalloc.h"
#include "obsproc.h"
#include "stdinclude.h"
#include "common.h"
#include "pending.h"
#include "playerdb.h"
#include "gamedb.h"
#include "board.h"
#include "utils.h"
#include "ratings.h"
#include <string.h>

char *colorstr[] = {"", "[black] ", "[white] "};
char *adjustr[] = {"", " (adjourned)"};

PRIVATE void prepare_match(int g,int wt,int bt,int winc,int binc,int wp, int bp,int rated)
{

  wt = wt * 60;                 /* To Seconds */
  bt = bt * 60;
  garray[g].white = wp;
  garray[g].black = bp;
  strcpy(garray[g].white_name, parray[wp].name);
  strcpy(garray[g].black_name, parray[bp].name);
  garray[g].status = GAME_ACTIVE;
  if ((garray[g].type == TYPE_UNTIMED) || (garray[g].type == TYPE_NONSTANDARD))
    garray[g].rated = 0;
  else
    garray[g].rated = rated;
  garray[g].private = BoolCheckPFlag(wp, PFLAG_PRIVATE)
                      || BoolCheckPFlag(bp, PFLAG_PRIVATE);
  garray[g].white = wp;
  if (garray[g].type == TYPE_BLITZ) {
    garray[g].white_rating = parray[wp].b_stats.rating;
    garray[g].black_rating = parray[bp].b_stats.rating;
  } else if (garray[g].type == TYPE_WILD) {
    garray[g].white_rating = parray[wp].w_stats.rating;
    garray[g].black_rating = parray[bp].w_stats.rating;
  } else if (garray[g].type == TYPE_LIGHT) {
    garray[g].white_rating = parray[wp].l_stats.rating;
    garray[g].black_rating = parray[bp].l_stats.rating;
  } else if (garray[g].type == TYPE_BUGHOUSE) {
    garray[g].white_rating = parray[wp].bug_stats.rating;
    garray[g].black_rating = parray[bp].bug_stats.rating;
  } else {
    garray[g].white_rating = parray[wp].s_stats.rating;
    garray[g].black_rating = parray[bp].s_stats.rating;
  }

  garray[g].game_state.gameNum = g;

  garray[g].wTime = wt * 10;
  garray[g].wInitTime = wt * 10;
  garray[g].wIncrement = winc * 10;
  garray[g].bTime = bt * 10;

  if (garray[g].type != TYPE_UNTIMED) {
    if (wt == 0)
        garray[g].wTime = 100;
    if (bt == 0)
        garray[g].bTime = 100;
  } /* 0 x games start with 10 seconds */

#ifdef TIMESEAL

  garray[g].wRealTime = garray[g].wTime * 100;
  garray[g].bRealTime = garray[g].bTime * 100;
  garray[g].wTimeWhenReceivedMove = 0;
  garray[g].bTimeWhenReceivedMove = 0;

#endif

  garray[g].bInitTime = bt * 10;
  garray[g].bIncrement = binc * 10;

  if (garray[g].game_state.onMove == BLACK) {   /* Start with black */
    garray[g].numHalfMoves = 1;
    garray[g].moveListSize = 1;
    garray[g].moveList = (move_t *) rmalloc(sizeof(move_t));
    garray[g].moveList[0].fromFile = -1;
    garray[g].moveList[0].fromRank = -1;
    garray[g].moveList[0].toFile = -1;
    garray[g].moveList[0].toRank = -1;
    garray[g].moveList[0].color = WHITE;
    strcpy(garray[g].moveList[0].moveString, "NONE");
    strcpy(garray[g].moveList[0].algString, "NONE");
  } else {
    garray[g].numHalfMoves = 0;
    garray[g].moveListSize = 0;
    garray[g].moveList = NULL;
  }

  garray[g].timeOfStart = tenth_secs();
  garray[g].startTime = tenth_secs();
  garray[g].lastMoveTime = garray[g].startTime;
  garray[g].lastDecTime = garray[g].startTime;
  garray[g].clockStopped = 0;

}

PRIVATE void pick_colors(int* wp,int* bp,int white,int wt,int bt,int winc,
								int binc)

{
 int reverse = 0;

  if (white == 0) {
    reverse = 1; /* did challenger ask for black? */

  } else if (white == -1) { /* unknown */

    if ((wt == bt) && (winc == binc)) { /* if diff times challenger is white */

      if (CheckPFlag(*wp, PFLAG_LASTBLACK)==CheckPFlag(*bp, PFLAG_LASTBLACK)) {
        if ((parray[*wp].num_white - parray[*wp].num_black) >
          (parray[*bp].num_white - parray[*bp].num_black))
          reverse = 1; /* whose played the most extra whites gets black */

      } else if (!CheckPFlag(*wp, PFLAG_LASTBLACK))
        reverse = 1;

    } else
      reverse = 1;              
  }
  if (reverse) {
    int tmp = *wp;
    *wp = *bp;
    *bp = tmp;
  }
}

PRIVATE void output_match_messages(int wp,int bp,int g, char* mess)
{

  int printed;
  int avail_printed;
  char outStr[1024];
  char notavail_white[200];
  char notavail_black[200];
  int p;

  notavail_white[0] = '\0';
  notavail_black[0] = '\0';

  sprintf(outStr, "\n{Game %d (%s vs. %s) %s %s %s match.}\n",
          g + 1, parray[wp].name,
          parray[bp].name,
	  mess,
          rstr[garray[g].rated],
          bstr[garray[g].type]);
  pprintf(wp, "%s", outStr);
  pprintf(bp, "%s", outStr);

  if (!(player_num_active_boards(wp)) && CheckPFlag(wp, PFLAG_OPEN))
/* open may be 0 if a simul */
    getnotavailmess(wp,notavail_white);
  if (CheckPFlag(bp, PFLAG_OPEN))
    getnotavailmess(bp,notavail_black);
  for (p = 0; p < p_num; p++) {
    int gnw, gnb;
    printed = 0;

    if ((p == wp) || (p == bp))
      continue;
    if (parray[p].status != PLAYER_PROMPT)
      continue;
    if (CheckPFlag(p, PFLAG_GIN)) {
      pprintf(p, "%s", outStr);
      printed = 1;
    }
    gnw = in_list(p, L_GNOTIFY, parray[wp].login);
    gnb = in_list(p, L_GNOTIFY, parray[bp].login);
    if (gnw || gnb) {
      pprintf(p, "\nGame notification: ");

      if (gnw)
        pprintf_highlight(p, parray[wp].name);
      else
        pprintf(p, parray[wp].name);

      pprintf(p, " (%s) vs. ",
              ratstr(GetRating(&parray[wp], garray[g].type)));

      if (gnb)
        pprintf_highlight(p, parray[bp].name);
      else
        pprintf(p, parray[bp].name);
      pprintf(p, " (%s) %s %s %d %d: Game %d\n",
                     ratstr(GetRating(&parray[bp], garray[g].type)),
                     rstr[garray[g].rated], bstr[garray[g].type],
                     garray[g].wInitTime/600, garray[g].wIncrement/10, g+1);

      printed = 1;
    }

    if (CheckPFlag(p, PFLAG_AVAIL) && CheckPFlag(p, PFLAG_OPEN) && (parray[p].game < 0) &&
       (CheckPFlag(wp, PFLAG_OPEN) || CheckPFlag(bp, PFLAG_OPEN))) {

      if (((parray[wp].b_stats.rating <= parray[p].availmax) &&
         (parray[wp].b_stats.rating >= parray[p].availmin)) ||
         (!parray[p].availmax)) {
         pprintf (p,"\n%s",notavail_white);
         avail_printed = 1;
      }

      if (((parray[bp].b_stats.rating <= parray[p].availmax) &&
         (parray[bp].b_stats.rating >= parray[p].availmin)) ||
         (!parray[p].availmax)) {
      pprintf (p,"\n%s",notavail_black);
      avail_printed = 1;
      }

      if (avail_printed) {
        printed = 1;
        avail_printed = 0;
        pprintf (p,"\n");
      } /* was black or white open originally (for simuls) */
    }

    if (printed)
      send_prompt (p);
  }
}

PUBLIC int create_new_match(int g, int white_player, int black_player,
                             int wt, int winc, int bt, int binc,
                             int rated, char *category, char *board,
                             int white, int simul)
{

  remove_request(white_player, black_player, -PEND_PARTNER);

  pick_colors(&white_player,&black_player,white,wt,bt,winc,binc);

  decline_withdraw_offers(white_player,-1, PEND_MATCH,DO_WITHDRAW | DO_DECLINE);
  decline_withdraw_offers(black_player,-1, PEND_MATCH,DO_WITHDRAW | DO_DECLINE);
  decline_withdraw_offers(black_player,-1, PEND_SIMUL,DO_WITHDRAW | DO_DECLINE);
  if (simul) {
    decline_withdraw_offers(white_player, -1, PEND_SIMUL, DO_WITHDRAW);
  } else {
    decline_withdraw_offers(white_player, -1, PEND_SIMUL,
                            DO_WITHDRAW | DO_DECLINE);
  }
  if (board_init(g,&garray[g].game_state, category, board)) {
    pprintf(white_player, "PROBLEM LOADING BOARD. Game Aborted.\n");
    pprintf(black_player, "PROBLEM LOADING BOARD. Game Aborted.\n");
    fprintf(stderr, "FICS: PROBLEM LOADING BOARD %s %s. Game Aborted.\n",
            category, board);
    game_remove(g);
    return COM_FAILED;
  }

  if (simul)
    garray[g].type = TYPE_UNTIMED;
  else
    garray[g].type = game_isblitz(wt, winc, bt, binc, category, board);

  prepare_match(g,wt,bt,winc,binc,white_player,black_player,rated);

  output_match_messages(white_player,black_player,g, "Creating");

  parray[white_player].game = g;
  parray[white_player].opponent = black_player;
  parray[white_player].side = WHITE;
  parray[white_player].promote = QUEEN;
  parray[black_player].game = g;
  parray[black_player].opponent = white_player;
  parray[black_player].side = BLACK;
  parray[black_player].promote = QUEEN;
  send_boards(g);
  userstat.games++;

  return COM_OK;
}
 
PUBLIC int accept_match(pending* pend, int p, int p1)
{
  int wt, winc, bt, binc, rated, white;
  char category[50], board[50];
  char tmp[100];
  int bh=0, pp, pp1,g1,g2;

  unobserveAll(p);              /* stop observing when match starts */
  unobserveAll(p1);

  wt = pend->param1;
  winc = pend->param2;
  bt = pend->param3;
  binc = pend->param4;
  rated = pend->param5;
  strcpy (category, pend->char1);
  strcpy (board, pend->char2);
  white = (pend->param6 == -1) ? -1 : 1 - pend->param6;

  pprintf(p, "You accept the challenge of %s.\n", parray[p1].name);
  pprintf_prompt(p1, "\n%s accepts your challenge.\n", parray[p].name);

  delete_pending(pend);

  pend_join_match (p,p1);

  if (game_isblitz(wt, winc, bt, binc, category, board) == TYPE_BUGHOUSE) {
    bh = 1;

    if ((pp = parray[p].partner) >= 0 &&
        (pp1 = parray[p1].partner) >= 0) {
      unobserveAll(pp);         /* stop observing when match starts */
      unobserveAll(pp1);

      pprintf(pp, "\nYour partner accepts the challenge of %s.\n", parray[p1].name);
      pprintf(pp1, "\n%s accepts your partner's challenge.\n", parray[p].name);

  pend_join_match (pp,pp1);
    } else {
      return COM_OK;
    }
  }

  g1 = game_new(); /* create a game structure */
  if (g1 < 0) {
    sprintf(tmp, "There was a problem creating the new match.\n");
    pprintf(p, tmp);
    pprintf_prompt(p1, tmp);
  }

  if (game_read(g1, p1, p) >= 0) {
    int swap;
    swap = p;
    p = p1;
    p1 = swap;

  } else if (game_read(g1, p, p1) < 0) { /* so no adjourned game */ 
    if (create_new_match(g1,p, p1, wt, winc, bt, binc, rated, category, board, white,0) == COM_FAILED)
      return COM_OK;

 /* create first game */

    if (bh) { /* do bughouse creation */

      white = (parray[p].side == WHITE ? 0 : 1);
      g2 = game_new();
      if ((g2 < 0) || (create_new_match(g2,pp, pp1, wt, winc, bt, binc, rated, category, board,white,0) == COM_FAILED)) {
         sprintf(tmp, "There was a problem creating the new match.\n");
         pprintf_prompt(pp, tmp);
         pprintf_prompt(pp1, tmp);
         sprintf(tmp, "There was a problem creating your partner's match.\n");
         game_remove(g1); /* abort first game */
         return COM_OK;
       }

      garray[g1].link = g2; /* link the games */
      garray[g2].link = g1;

      sprintf(tmp, "\nYour partner is playing game %d (%s vs. %s).\n",
              g2 + 1, garray[g2].white_name, garray[g2].black_name);
      pprintf(p, tmp);
      pprintf_prompt(p1, tmp);

      sprintf(tmp, "\nYour partner is playing game %d (%s vs. %s).\n",
              g1 + 1, garray[g1].white_name, garray[g1].black_name);
      pprintf_prompt(pp, tmp);
      pprintf_prompt(pp1, tmp);
    } 
    return COM_OK;
  }

       /* resume adjourned game */

  game_delete(p, p1); /* remove the game from disk */

  sprintf(tmp, "{Game %d (%s vs. %s) Continuing %s %s match.}\n",
        g1+1, parray[p].name, parray[p1].name,
        rstr[garray[g1].rated], bstr[garray[g1].type]);
  pprintf(p, tmp);
  pprintf(p1, tmp);
  output_match_messages(p, p1, g1, "Continuing");

  garray[g1].white = p;
  garray[g1].black = p1;
  garray[g1].status = GAME_ACTIVE;
  garray[g1].result = END_NOTENDED;
  garray[g1].startTime = tenth_secs();
  garray[g1].lastMoveTime = garray[g1].startTime;
  garray[g1].lastDecTime = garray[g1].startTime;
  parray[p].game = g1;
  parray[p].opponent = p1;
  parray[p].side = WHITE;
  parray[p1].game = g1;
  parray[p1].opponent = p;
  parray[p1].side = BLACK;

#ifdef TIMESEAL
  garray[g1].wRealTime = garray[g1].wTime * 100;
  garray[g1].bRealTime = garray[g1].bTime * 100;
  garray[g1].wTimeWhenReceivedMove = 0;
  garray[g1].bTimeWhenReceivedMove = 0;
#endif

  send_boards(g1);
  return COM_OK;
}

/* board and category are initially empty strings */
PRIVATE int parse_match_string(int p, int* wt,int* bt,int* winc,int* binc,
                                int* white,int* rated,char* category,
			        char* board, char* mstring)
{
  int numba;
  int confused = 0;
  char parsebuf[100];
  int bughouse = 0;

  while (!confused && (sscanf(mstring, " %99s", parsebuf) == 1)) {
    mstring = eatword(eatwhite(mstring));

    if ((category[0] != '\0') && (board[0] == '\0') && !bughouse)
      strcpy(board, parsebuf);
    else if (isdigit(*parsebuf)) {
      if ((numba = atoi(parsebuf)) < 0) {
        pprintf(p, "You can't specify negative time controls.\n");
        return 0;
      } else if (numba > 999) {
        pprintf(p, "You can't specify time or inc above 999.\n");
        return 0;
      } else if (*wt == -1) {
        *wt = numba;
      } else if (*winc == -1) {
        *winc = numba;
      } else if (*bt == -1) {
        *bt = numba;
      } else if (*binc == -1) {
        *binc = numba;
      } else
        confused = 1;

    } else if ((!strcmp("rated", parsebuf)) || (!strcmp("r",parsebuf))) {
      if (*rated == -1)
        *rated = 1;
      else
        confused = 1;
    } else if ((!strcmp("unrated", parsebuf)) || (!strcmp("u",parsebuf))) {
      if (*rated == -1)
        *rated = 0;
      else
        confused = 1;
    } else if (!strcmp("white", parsebuf) || !strcmp ("w", parsebuf)) {
      if (*white == -1)
        *white = 1;
      else
        confused = 1;
    } else if (!strcmp("black", parsebuf) || !strcmp ("b", parsebuf)) {
      if (*white == -1)
        *white = 0;
      else
        confused = 1;

    } else if (category[0] == '\0') {
      if ((parsebuf[0] == 'w') && (isdigit(*(parsebuf + 1)))) {
        strcpy (category, "wild");
        strcpy (board, parsebuf+1);
      } else {
        strcpy(category,parsebuf);
        if (!strncmp("bughouse",category, strlen(category))
            && strlen(category) >= 3) {
          strcpy(category, "bughouse");
          bughouse = 1;
        }
      }
    } else
      confused = 1;
  }

  if (confused) {
    pprintf(p, "Can't interpret %s in match command.\n", parsebuf);
    return 0;
  }
  return 1;
}

PRIVATE void print_match_rating_info(int p,int p1,int type,int rated)
{
  if (rated) {
    int win, draw, loss;
    double newsterr;

    rating_sterr_delta(p1, p, type, time(0), RESULT_WIN, &win, &newsterr);
    rating_sterr_delta(p1, p, type, time(0), RESULT_DRAW, &draw, &newsterr);
    rating_sterr_delta(p1, p, type, time(0), RESULT_LOSS, &loss, &newsterr);
    pprintf(p1, "Your %s rating will change:  Win: %s%d,  Draw: %s%d,  Loss: %s%d\n",
            bstr[type],
            (win >= 0) ? "+" : "", win,
            (draw >= 0) ? "+" : "", draw,
            (loss >= 0) ? "+" : "", loss);
    pprintf(p1, "Your new RD will be %5.1f\n", newsterr);

    rating_sterr_delta(p, p1, type, time(0), RESULT_WIN, &win, &newsterr);
    rating_sterr_delta(p, p1, type, time(0), RESULT_DRAW, &draw, &newsterr);
    rating_sterr_delta(p, p1, type, time(0), RESULT_LOSS, &loss, &newsterr);
    pprintf(p, "Your %s rating will change:  Win: %s%d,  Draw: %s%d,  Loss: %s%d\n",
            bstr[type],
            (win >= 0) ? "+" : "", win,
            (draw >= 0) ? "+" : "", draw,
            (loss >= 0) ? "+" : "", loss);
    pprintf(p, "Your new RD will be %5.1f\n", newsterr);
  }
}

PRIVATE void check_lists_match(int p,int p1)
{
  if (in_list(p, L_COMPUTER, parray[p].name)) {
    pprintf(p1, "--** %s is a ", parray[p].name);
    pprintf_highlight(p1, "computer");
    pprintf(p1, " **--\n");
  }
  if (in_list(p, L_COMPUTER, parray[p1].name)) {
    pprintf(p, "--** %s is a ", parray[p1].name);
    pprintf_highlight(p, "computer");
    pprintf(p, " **--\n");
  }
  if (in_list(p, L_ABUSER, parray[p].name)) {
    pprintf(p1, "--** %s is in the ", parray[p].name);
    pprintf_highlight(p1, "abuser");
    pprintf(p1, " list **--\n");
  }
  if (in_list(p, L_ABUSER, parray[p1].name)) {
    pprintf(p, "--** %s is in the ", parray[p1].name);
    pprintf_highlight(p, "abuser");
    pprintf(p, " list **--\n");
  }
}

PUBLIC int com_match(int p, param_list param)
{
  int adjourned;                /* adjourned game? */
  int g;                        /* more adjourned game junk */
  int p1;
  int bh=0, pp, pp1;
  pending* pendfrom;
  pending* pendto;
  pending* pend;
  int wt = -1;                  /* white start time */
  int winc = -1;                /* white increment */
  int bt = -1;                  /* black start time */
  int binc = -1;                /* black increment */
  int rated = -1;               /* 1 = rated, 0 = unrated */
  int white = -1;               /* 1 = want white, 0 = want black */
  char category[100], board[100];
  textlist *clauses = NULL;
  int type;

  category[0] ='\0';
  board[0] ='\0';

  if ((parray[p].game >= 0) && ((garray[parray[p].game].status == GAME_EXAMINE)
|| (garray[parray[p].game].status == GAME_SETUP))) {

    pprintf(p, "You can't challenge while you are examining a game.\n");
    return COM_OK;
  }

  if (parray[p].game >= 0) {
    pprintf(p, "You can't challenge while you are playing a game.\n");
    return COM_OK;
  }

  stolower(param[0].val.word);
  p1 = player_find_part_login(param[0].val.word);
  if (p1 < 0) {
    pprintf(p, "No user named \"%s\" is logged in.\n", param[0].val.word);
    return COM_OK;
  }

  if (p1 == p) {                /* Allowing to match yourself to enter
                                   analysis mode */
    ExamineScratch (p, param, 0);
    return COM_OK;
  }

  if (!CheckPFlag(p1, PFLAG_OPEN)) {
    pprintf(p, "Player \"%s\" is not open to match requests.\n", parray[p1].name);
    return COM_OK;
  }

  if (parray[p1].game >= 0) {
    pprintf(p, "Player \"%s\" is involved in another game.\n", parray[p1].name);    return COM_OK;
  }

  if (CheckPFlag(p, PFLAG_TOURNEY) && !CheckPFlag(p1, PFLAG_TOURNEY)) {
    pprintf(p, "You may only match players with their tournament variable set.\n");
    return COM_OK;
  }

  if (!CheckPFlag(p, PFLAG_TOURNEY) && CheckPFlag(p1, PFLAG_TOURNEY)) {
    pprintf(p, "%s is in a tournament, and cannot accept other challenges.\n", parray[p1].name);
    return COM_OK;
  }

  if (!CheckPFlag(p, PFLAG_OPEN)) {
    PFlagON(p, PFLAG_OPEN);
    pprintf(p, "Setting you open for matches.\n");
  }


/* look for an adjourned game between p and p1 */
  g = game_new();
  adjourned = (game_read(g, p, p1) >= 0) || (game_read(g, p1, p) >= 0);
  if (adjourned) {
    type = garray[g].type;
    wt = garray[g].wInitTime / 600;
    bt = garray[g].bInitTime / 600;
    winc = garray[g].wIncrement / 10;
    binc = garray[g].bIncrement / 10;
    rated = garray[g].rated;
  }
  game_remove(g);

  pendto = find_pend(p, p1, PEND_MATCH);

  pendfrom = find_pend(p1, p, PEND_MATCH);
 
  if (!adjourned) {
    if (in_list(p1, L_NOPLAY, parray[p].login)) {
      pprintf(p, "You are on %s's noplay list.\n", parray[p1].name);
      return COM_OK;
    }
    if (player_censored(p1, p)) {
      pprintf(p, "Player \"%s\" is censoring you.\n", parray[p1].name);
      return COM_OK;
    }
    if (player_censored(p, p1)) {
      pprintf(p, "You are censoring \"%s\".\n", parray[p1].name);
      return COM_OK;
    }

    if (param[1].type != TYPE_NULL) {
      if (!parse_match_string(p, &wt,&bt,&winc,&binc,&white,&rated,category,
                                                  board,param[1].val.string))
      return COM_OK; /* couldn't parse */
    }

    if (rated == -1)
      rated = BoolCheckPFlag(p, PFLAG_RATED);
    if (!CheckPFlag(p, PFLAG_REG) || !CheckPFlag(p1, PFLAG_REG))
      rated = 0;

    if (winc == -1)
      winc = (wt == -1) ? parray[p].d_inc : 0;  /* match 5 == match 5 0 */

    if (wt == -1)
      wt = parray[p].d_time;

    if (bt == -1)
      bt = wt;

    if (binc == -1)
      binc = winc;

    if (category[0]) {
      if (!board[0] && strcmp(category,"bughouse")) {
        pprintf(p, "You must specify a board and a category.\n");
        return COM_OK;

      } else if (board[0]) { /* not bughouse */
        char fname[MAX_FILENAME_SIZE];

        sprintf(fname, "%s/%s/%s", board_dir, category, board);
        if (!file_exists(fname)) {
          pprintf(p, "No such category/board: %s/%s\n", category, board);
          return COM_OK;
        }
      }
    }
    type = game_isblitz(wt, winc, bt, binc, category, board);

    if (!strcmp(category, "bughouse")) {
      type = TYPE_BUGHOUSE;
      if (rated && parray[p].partner >= 0 && parray[p1].partner >= 0) {
        if (!CheckPFlag(parray[p].partner, PFLAG_REG)
              || !CheckPFlag(parray[p1].partner, PFLAG_REG))
          rated = 0;
      }
    }
    if (rated && (type == TYPE_NONSTANDARD)) {
      pprintf(p, "Game is non-standard - reverting to unrated\n");
      rated = 0;
    }
    if (rated && (type == TYPE_UNTIMED)) {
      pprintf(p, "Game is untimed - reverting to unrated\n");
      rated = 0;
    }
    if ((pendfrom == NULL) && !CheckPFlag(p1, PFLAG_ROPEN)
        && (rated != BoolCheckPFlag(p1, PFLAG_RATED))) {
      pprintf(p, "%s only wants to play %s games.\n", parray[p1].name,
              rstr[!rated]);
      pprintf_highlight(p1, "Ignoring");
      pprintf(p1, " %srated match request from %s.\n",
              (rated ? "" : "un"), parray[p].name);
      return COM_OK;
    }

    /* Now check formula. */
    if (!adjourned
        && !GameMatchesFormula(p,p1, wt,winc,bt,binc, rated, type, &clauses)) {
      pprintf(p, "Match request does not fit formula for %s:\n",
              parray[p1].name);
      pprintf(p, "%s's formula: %s\n", parray[p1].name, parray[p1].formula);
      ShowClauses (p, p1, clauses);
      ClearTextList(clauses);
      pprintf_highlight(p1, "Ignoring");
      pprintf_prompt(p1, " (formula): %s (%d) %s (%d) %s.\n",
                     parray[p].name,
                     GetRating(&parray[p], type),
                     parray[p1].name,
                     GetRating(&parray[p1], type),
            game_str(rated, wt * 60, winc, bt * 60, binc, category, board));
      return COM_OK;
    }

    if (type == TYPE_BUGHOUSE) {
      bh = 1;
      pp = parray[p].partner;
      pp1 = parray[p1].partner;

      if (pp < 0) {
        pprintf(p, "You have no partner for bughouse.\n");
        return COM_OK;
      }
      if (pp1 < 0) {
        pprintf(p, "Your opponent has no partner for bughouse.\n");
        return COM_OK;
      }
      if (pp == pp1) { /* should be an impossible case - DAV */
        pprintf(p, "You and your opponent both chose the same partner!\n");
        return COM_OK;
      }
      if (pp == p1 || pp1 == p) {
        pprintf(p, "You and your opponent can't choose each other as partners!\n");
        return COM_OK;
      }
      if (parray[pp].partner != p) { /* another impossible case - DAV */
        pprintf(p, "Your partner hasn't chosen you as his partner!\n");
        return COM_OK;
      }
      if (parray[pp1].partner != p1) { /* another impossible case - DAV */
        pprintf(p, "Your opponent's partner hasn't chosen your opponent as his partner!\n");
        return COM_OK;
      }
      if (!CheckPFlag(pp, PFLAG_OPEN) || parray[pp].game >= 0) {
        pprintf(p, "Your partner isn't open to play right now.\n");
        return COM_OK;
      }
      if (!CheckPFlag(pp1, PFLAG_OPEN) || parray[pp1].game >= 0) {
        pprintf(p, "Your opponent's partner isn't open to play right now.\n");
        return COM_OK;
      }

      /* Bypass NOPLAY lists, censored lists, ratedness, privacy, and formula for now */
      /*  Active challenger/ee will determine these. */
    }
    /* Ok match offer will be made */

  }                             /* adjourned games shouldn't have to worry
                                   about that junk? */
				/* keep incase of adjourned bughouse in future*/

  if (pendto != NULL) {
    pprintf(p, "Updating offer already made to \"%s\".\n", parray[p1].name);
  }

  if (pendfrom != NULL) {
    if (pendto != NULL) {
      pprintf(p, "Pending list error!.\n");
      fprintf(stderr, "FICS: This shouldn't happen. You can't have a match pending from and to the same person.\n");
      return COM_OK;
    }

    if (adjourned || ((wt == pendfrom->param3) &&
                      (winc == pendfrom->param4) &&
                      (bt == pendfrom->param1) &&
                      (binc == pendfrom->param2) &&
                      (rated == pendfrom->param5) &&
                      ((white == -1) || (white + pendfrom->param6 == 1)) &&
               (!strcmp(category, pendfrom->char1)) &&
                 (!strcmp(board, pendfrom->char2)))) {
      /* Identical match, should accept! */
      accept_match(pendfrom,p, p1);
      return COM_OK;

    } else {
      delete_pending(pendfrom);
    }
  }

  if (pendto == NULL)
    pend = add_pending(p,p1,PEND_MATCH);
  else
    pend = pendto; 

  pend->param1 = wt;
  pend->param2 = winc;
  pend->param3 = bt;
  pend->param4 = binc;
  pend->param5 = rated;
  pend->param6 = white;
  pend->param7 = type;
  pend->char1 = rstrdup(category);
  pend->char2 = rstrdup (board);

  if (pendfrom != NULL) {
    pprintf(p, "Declining offer from %s and offering new match parameters.\n", parray[p1].name);
    pprintf(p1, "\n%s declines your match offer a match with these parameters:", parray[p].name);
  }

  if (pendto != NULL) {
    pprintf(p, "Updating match request to: ");
    pprintf(p1, "\n%s updates the match request.\n", parray[p].name);
  } else {
    pprintf(p, "Issuing: ");
    pprintf(p1, "\n", parray[p].name);
  }

  pprintf(p, "%s (%s) %s", parray[p].name,
          ratstrii(GetRating(&parray[p], type), p),
          colorstr[white + 1]);
  pprintf_highlight(p, "%s", parray[p1].name);
  pprintf(p, " (%s) %s%s.\n",
          ratstrii(GetRating(&parray[p1], type), p1),
          game_str(rated, wt * 60, winc, bt * 60, binc, category, board),
          adjustr[adjourned]);
  pprintf(p1, "Challenge: ");
  pprintf_highlight(p1, "%s", parray[p].name);
  pprintf(p1, " (%s) %s",
          ratstrii(GetRating(&parray[p], type), p),
          colorstr[white + 1]);
  pprintf(p1, "%s (%s) %s%s.\n", parray[p1].name,
          ratstrii(GetRating(&parray[p1], type), p1),
          game_str(rated, wt * 60, winc, bt * 60, binc, category, board),
          adjustr[adjourned]);
  Bell (p1);

  if (bh) {

    pprintf(pp, "\nYour bughouse partner issuing %s (%s) %s",
            parray[p].name, ratstrii(GetRating(&parray[p], type), p),
            colorstr[white + 1]);
    pprintf_highlight(pp, "%s", parray[p1].name);
    pprintf(pp, " (%s) %s.\n",
            ratstrii(GetRating(&parray[p1], type), p1),
            game_str(rated, wt * 60, winc, bt * 60, binc, category, board));
    pprintf(pp, "Your game would be ");
    pprintf_highlight(pp, "%s", parray[pp1].name);
    pprintf_prompt(pp, " (%s) %s%s (%s) %s.\n",
          ratstrii(GetRating(&parray[pp1], type), pp1),
          colorstr[white + 1], parray[pp].name,
          ratstrii(GetRating(&parray[pp], type), pp),
          game_str(rated, wt * 60, winc, bt * 60, binc, category, board));
    Bell (pp);

    pprintf(pp1, "\nYour bughouse partner was challenged ");
    pprintf_highlight(pp1, "%s", parray[p].name);
    pprintf(pp1, " (%s) %s", ratstrii(GetRating(&parray[p], type), p),
                             colorstr[white + 1]);
    pprintf(pp1, "%s (%s) %s.\n", parray[p1].name,
            ratstrii(GetRating(&parray[p1], type), p1),
            game_str(rated, wt * 60, winc, bt * 60, binc, category, board));
    pprintf(pp1, "Your game would be %s (%s) %s", parray[pp1].name,
          ratstrii(GetRating(&parray[pp1], type), pp1),
          colorstr[white + 1]);
    pprintf_highlight(pp1, "%s", parray[pp].name);
    pprintf_prompt(pp1, " (%s) %s.\n",
          ratstrii(GetRating(&parray[pp], type), pp),
          game_str(rated, wt * 60, winc, bt * 60, binc, category, board));
    Bell(pp1);
  }

  check_lists_match (p,p1);

  print_match_rating_info (p,p1,type,rated);

  pprintf_prompt(p1, "You can \"accept\" or \"decline\", or propose different parameters.\n");

  return COM_OK;
}

