/* rmalloc.c
 *
 */

/*
    fics - An internet chess server.
    Copyright (C) 1993  Richard V. Nash

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

/* Revision history:
   name		email		yy/mm/dd	Change
   Richard Nash	              	93/10/22	Created
*/


#include "stdinclude.h"

#include "common.h"

#include "rmalloc.h"

PUBLIC unsigned int allocated_size = 0;
PUBLIC unsigned int malloc_count = 0;
PUBLIC unsigned int free_count = 0;
PUBLIC unsigned int realloc_count = 0;
PUBLIC unsigned int strdup_count = 0;

PUBLIC unsigned int malloc_size_count[MAX_MALLOC_SIZE_COUNT];
PUBLIC unsigned int malloc_inuse_count[MAX_MALLOC_SIZE_COUNT];

/*
* hack by shane 
*/
#define malloc_usable_size(x) (0)

PUBLIC void *rmalloc(int byteSize)
{
  void *newptr;
#ifdef HASMALLOCSIZE
  allocated_size += byteSize;
#endif
  if (malloc_count == 0) {  /* initialize malloc_size_count array */
    int i;
    for (i=0; i < MAX_MALLOC_SIZE_COUNT; i++)
      malloc_size_count[i] = 0;
  }
  malloc_count++;
  newptr = malloc(byteSize);
  if (newptr == NULL) {
      fprintf(stderr, "Out of memory in malloc!\n");
      abort();
  }
  byteSize = malloc_usable_size(newptr);
  if (byteSize >= MAX_MALLOC_SIZE_COUNT) {
    malloc_size_count[MAX_MALLOC_SIZE_COUNT-1] +=1;
    malloc_inuse_count[MAX_MALLOC_SIZE_COUNT-1] +=1;
  } else {
    malloc_size_count[byteSize] += 1;
    malloc_inuse_count[byteSize] += 1;
  }

  return newptr;
}

PUBLIC void *rrealloc(void *ptr, int byteSize)
{
  int oldsize, newsize;
#ifdef HASMALLOCSIZE
  allocated_size += (byteSize - malloc_usable_size(ptr));
#endif
  if (ptr == NULL) {		/* Sparky  3/16/95  */
    fprintf(stderr, "Hoser!  Null ptr passed to rrealloc!!\n");
    return NULL;
  } else {
    void *newptr = realloc(ptr, byteSize);
    realloc_count++;
    if (newptr == NULL) {
	fprintf(stderr, "Out of memory in rrealloc!\n");
	abort();
    }
    oldsize = malloc_usable_size(ptr);
    newsize = malloc_usable_size(newptr);
    if (oldsize >= MAX_MALLOC_SIZE_COUNT) oldsize = MAX_MALLOC_SIZE_COUNT - 1;
    if (newsize >= MAX_MALLOC_SIZE_COUNT) newsize = MAX_MALLOC_SIZE_COUNT - 1;
    malloc_size_count[newsize] += 1;
    malloc_inuse_count[newsize] += 1;
    malloc_inuse_count[oldsize] -= 1;
    return newptr;
  }
}

PUBLIC void rfree(void *ptr)
{
#ifdef HASMALLOCSIZE
  allocated_size = allocated_size - malloc_usable_size(ptr);
#endif
  if (ptr == NULL) {		/* Sparky  3/16/95  */
    fprintf(stderr, "Hoser!  Null ptr passed to rfree!!\n");
  } else {
    int byteSize = malloc_usable_size(ptr);
    if ((byteSize >= MAX_MALLOC_SIZE_COUNT))
      malloc_inuse_count[MAX_MALLOC_SIZE_COUNT-1] -=1;
    else
      malloc_inuse_count[byteSize] -= 1;

    free_count++;
    free(ptr);
  }
}

PUBLIC void strfree (char *string)
{
  /* This routine is often called on strings that are either
     a rmalloc'd value or NULL.  So it's not an error to see
     a NULL here.  --mann
  */
  if (string != NULL) rfree (string);
}
